/*
 * 
 * Copyright (c) 2007, Justin Etheredge <justin@etheredge.us> http://www.codethinked.com
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 * 
 * Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 * 
 * Neither the name of CodeThinked nor the names of its contributors may be 
 * used to endorse or promote products derived from this software without 
 * specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

using System;
using System.ComponentModel;
using System.Web;
using BlogEngine.Core.Web.Controls;
using BlogEngine.Core;
using Joel.Net;


/// <summary>
/// Summary description for Askimet
/// </summary>
[Extension("   Askimet  . !    API key : <a href=\"http://wordpress.com/api-keys/\">wordpress.com</a>", "1.4", "<a href=\"http://www.codethinked.com\">Justin Etheredge</a>")]
public class AkismetExtension
{
    static protected ExtensionSettings _settings = null;
    
    // Constructor
    public AkismetExtension()
    {
        // Add the event handler for the CommentAdded event
        Post.AddingComment += new EventHandler<CancelEventArgs>(Post_AddingComment);

        ExtensionSettings settings = new ExtensionSettings("AkismetExtension");
        settings.AddParameter("apiKey", "API Key");
        settings.AddParameter("blogUrl", "Blog URL");
        settings.IsScalar = true;
        ExtensionManager.ImportSettings(settings);
        _settings = ExtensionManager.GetSettings("AkismetExtension");
    }       
    
    /// <summary>
    /// Handles the AddingComment event
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void Post_AddingComment(object sender, CancelEventArgs e)
    {
        Comment comment = (Comment)sender;        

        string apiKey = _settings.GetSingleValue("apiKey");
        string blogUrl = _settings.GetSingleValue("blogUrl");

        Akismet api = new Akismet(apiKey, blogUrl, "BlogEngine.net 1.4");
        if (!api.VerifyKey())
        {
            throw new ApplicationException("You must enter your blog information into the extensions manager in the admin section.");
        }
        AkismetComment askimetComment = new AkismetComment();        
        askimetComment.Blog = blogUrl;
        askimetComment.UserIp = comment.IP;
        askimetComment.CommentContent = comment.Content;
        askimetComment.CommentType = "comment";
        askimetComment.CommentAuthor = comment.Author;
        askimetComment.CommentAuthorEmail = comment.Email;
        if (comment.Website != null)
        {
            askimetComment.CommentAuthorUrl = comment.Website.OriginalString;    
        }
        if (api.CommentCheck(askimetComment))
        {            
            e.Cancel = true;
        }
    }

}